@@ -1,5 +1,5 @@
 //
-// Copyright 2020 Staysail Systems, Inc. <info@staysail.tech>
+// Copyright 2024 Staysail Systems, Inc. <info@staysail.tech>
 // Copyright 2018 Capitar IT Group BV <info@capitar.com>
 // Copyright 2018 Devolutions <info@devolutions.net>
 //
@@ -10,17 +10,19 @@
 //
 
 #include "core/nng_impl.h"
+#include "nng/nng.h"
 #include "sockimpl.h"
 
 #include <stdio.h>
 
-// This file contains functions relating to pipes.
+// This file contains functions related to pipe objects.
 //
 // Operations on pipes (to the transport) are generally blocking operations,
 // performed in the context of the protocol.
 
-static nni_id_map pipes;
-static nni_mtx    pipes_lk;
+static nni_id_map pipes =
+    NNI_ID_MAP_INITIALIZER(1, 0x7fffffff, NNI_ID_FLAG_RANDOM);
+static nni_mtx pipes_lk = NNI_MTX_INITIALIZER;
 
 static void pipe_destroy(void *);
 
@@ -29,26 +31,6 @@ static nni_reap_list pipe_reap_list = {
 	.rl_func   = pipe_destroy,
 };
 
-int
-nni_pipe_sys_init(void)
-{
-	nni_mtx_init(&pipes_lk);
-
-	// Pipe IDs needs to have high order bit clear, and we want
-	// them to start at a random value.
-	nni_id_map_init(&pipes, 1, 0x7fffffff, true);
-
-	return (0);
-}
-
-void
-nni_pipe_sys_fini(void)
-{
-	nni_reap_drain();
-	nni_mtx_fini(&pipes_lk);
-	nni_id_map_fini(&pipes);
-}
-
 static void
 pipe_destroy(void *arg)
 {
@@ -90,7 +72,6 @@ pipe_destroy(void *arg)
 		p->p_tran_ops.p_fini(p->p_tran_data);
 	}
 	nni_cv_fini(&p->p_cv);
-	nni_mtx_fini(&p->p_mtx);
 	nni_free(p, p->p_size);
 }
 
@@ -148,14 +129,9 @@ nni_pipe_send(nni_pipe *p, nni_aio *aio)
 void
 nni_pipe_close(nni_pipe *p)
 {
-	nni_mtx_lock(&p->p_mtx);
-	if (p->p_closed) {
-		// We already did a close.
-		nni_mtx_unlock(&p->p_mtx);
-		return;
+	if (nni_atomic_swap_bool(&p->p_closed, true)) {
+		return; // We already did a close.
 	}
-	p->p_closed = true;
-	nni_mtx_unlock(&p->p_mtx);
 
 	if (p->p_proto_data != NULL) {
 		p->p_proto_ops.pipe_close(p->p_proto_data);
@@ -169,6 +145,12 @@ nni_pipe_close(nni_pipe *p)
 	nni_reap(&pipe_reap_list, p);
 }
 
+bool
+nni_pipe_is_closed(nni_pipe *p)
+{
+	return (nni_atomic_get_bool(&p->p_closed));
+}
+
 uint16_t
 nni_pipe_peer(nni_pipe *p)
 {
@@ -245,51 +227,48 @@ pipe_stats_init(nni_pipe *p)
 #endif // NNG_ENABLE_STATS
 
 static int
-pipe_create(nni_pipe **pp, nni_sock *sock, nni_sp_tran *tran, void *tdata)
+pipe_create(nni_pipe **pp, nni_sock *sock, nni_sp_tran *tran, void *tran_data)
 {
-	nni_pipe *          p;
+	nni_pipe           *p;
 	int                 rv;
-	void *              sdata = nni_sock_proto_data(sock);
-	nni_proto_pipe_ops *pops  = nni_sock_proto_pipe_ops(sock);
+	void               *sock_data = nni_sock_proto_data(sock);
+	nni_proto_pipe_ops *pops      = nni_sock_proto_pipe_ops(sock);
 	size_t              sz;
 
 	sz = NNI_ALIGN_UP(sizeof(*p)) + pops->pipe_size;
 
 	if ((p = nni_zalloc(sz)) == NULL) {
 		// In this case we just toss the pipe...
-		tran->tran_pipe->p_fini(tdata);
+		tran->tran_pipe->p_fini(tran_data);
 		return (NNG_ENOMEM);
 	}
 
 	p->p_size       = sz;
 	p->p_proto_data = p + 1;
 	p->p_tran_ops   = *tran->tran_pipe;
-	p->p_tran_data  = tdata;
+	p->p_tran_data  = tran_data;
 	p->p_proto_ops  = *pops;
 	p->p_sock       = sock;
-	p->p_closed     = false;
 	p->p_cbs        = false;
-	p->p_ref        = 0;
+	p->p_ref        = 1;
 
+	nni_atomic_init_bool(&p->p_closed);
 	nni_atomic_flag_reset(&p->p_stop);
 	NNI_LIST_NODE_INIT(&p->p_sock_node);
 	NNI_LIST_NODE_INIT(&p->p_ep_node);
 
-	nni_mtx_init(&p->p_mtx);
 	nni_cv_init(&p->p_cv, &pipes_lk);
 
 	nni_mtx_lock(&pipes_lk);
-	if ((rv = nni_id_alloc(&pipes, &p->p_id, p)) == 0) {
-		p->p_ref = 1;
-	}
+	rv = nni_id_alloc32(&pipes, &p->p_id, p);
 	nni_mtx_unlock(&pipes_lk);
 
 #ifdef NNG_ENABLE_STATS
 	pipe_stats_init(p);
 #endif
 
-	if ((rv != 0) || ((rv = p->p_tran_ops.p_init(tdata, p)) != 0) ||
-	    ((rv = pops->pipe_init(p->p_proto_data, p, sdata)) != 0)) {
+	if ((rv != 0) || ((rv = p->p_tran_ops.p_init(tran_data, p)) != 0) ||
+	    ((rv = pops->pipe_init(p->p_proto_data, p, sock_data)) != 0)) {
 		nni_pipe_close(p);
 		nni_pipe_rele(p);
 		return (rv);
@@ -300,13 +279,13 @@ pipe_create(nni_pipe **pp, nni_sock *sock, nni_sp_tran *tran, void *tdata)
 }
 
 int
-nni_pipe_create_dialer(nni_pipe **pp, nni_dialer *d, void *tdata)
+nni_pipe_create_dialer(nni_pipe **pp, nni_dialer *d, void *tran_data)
 {
 	int          rv;
 	nni_sp_tran *tran = d->d_tran;
-	nni_pipe *   p;
+	nni_pipe    *p;
 
-	if ((rv = pipe_create(&p, d->d_sock, tran, tdata)) != 0) {
+	if ((rv = pipe_create(&p, d->d_sock, tran, tran_data)) != 0) {
 		return (rv);
 	}
 	p->p_dialer = d;
@@ -324,17 +303,17 @@ nni_pipe_create_dialer(nni_pipe **pp, nni_dialer *d, void *tdata)
 }
 
 int
-nni_pipe_create_listener(nni_pipe **pp, nni_listener *l, void *tdata)
+nni_pipe_create_listener(nni_pipe **pp, nni_listener *l, void *tran_data)
 {
 	int          rv;
 	nni_sp_tran *tran = l->l_tran;
-	nni_pipe *   p;
+	nni_pipe    *p;
 
-	if ((rv = pipe_create(&p, l->l_sock, tran, tdata)) != 0) {
+	if ((rv = pipe_create(&p, l->l_sock, tran, tran_data)) != 0) {
 		return (rv);
 	}
 	p->p_listener = l;
-#if NNG_ENABLE_STATS
+#ifdef NNG_ENABLE_STATS
 	static const nni_stat_info listener_info = {
 		.si_name = "listener",
 		.si_desc = "listener for pipe",
@@ -399,26 +378,26 @@ nni_pipe_add_stat(nni_pipe *p, nni_stat_item *item)
 }
 
 void
-nni_pipe_bump_rx(nni_pipe *p, size_t nbytes)
+nni_pipe_bump_rx(nni_pipe *p, size_t bytes)
 {
 #ifdef NNG_ENABLE_STATS
-	nni_stat_inc(&p->st_rx_bytes, nbytes);
+	nni_stat_inc(&p->st_rx_bytes, bytes);
 	nni_stat_inc(&p->st_rx_msgs, 1);
 #else
 	NNI_ARG_UNUSED(p);
-	NNI_ARG_UNUSED(nbytes);
+	NNI_ARG_UNUSED(bytes);
 #endif
 }
 
 void
-nni_pipe_bump_tx(nni_pipe *p, size_t nbytes)
+nni_pipe_bump_tx(nni_pipe *p, size_t bytes)
 {
 #ifdef NNG_ENABLE_STATS
-	nni_stat_inc(&p->st_tx_bytes, nbytes);
+	nni_stat_inc(&p->st_tx_bytes, bytes);
 	nni_stat_inc(&p->st_tx_msgs, 1);
 #else
 	NNI_ARG_UNUSED(p);
-	NNI_ARG_UNUSED(nbytes);
+	NNI_ARG_UNUSED(bytes);
 #endif
 }
 
@@ -430,4 +409,15 @@ nni_pipe_bump_error(nni_pipe *p, int err)
 	} else {
 		nni_listener_bump_error(p->p_listener, err);
 	}
-}
\ No newline at end of file
+}
+
+char *
+nni_pipe_peer_addr(nni_pipe *p, char buf[NNG_MAXADDRSTRLEN])
+{
+	nng_sockaddr sa;
+	size_t       sz = sizeof(sa);
+	sa.s_family     = AF_UNSPEC;
+	nni_pipe_getopt(p, NNG_OPT_REMADDR, &sa, &sz, NNI_TYPE_SOCKADDR);
+	nng_str_sockaddr(&sa, buf, NNG_MAXADDRSTRLEN);
+	return (buf);
+}
