@@ -1,5 +1,5 @@
 //
-// Copyright 2023 Staysail Systems, Inc. <info@staysail.tech>
+// Copyright 2024 Staysail Systems, Inc. <info@staysail.tech>
 // Copyright 2018 Capitar IT Group BV <info@capitar.com>
 //
 // This software is supplied under the terms of the MIT License, a
@@ -9,7 +9,9 @@
 //
 
 #include "core/nng_impl.h"
+#include "core/pipe.h"
 #include "list.h"
+#include "nng/nng.h"
 #include "sockimpl.h"
 
 #include <stdio.h>
@@ -940,7 +942,7 @@ int
 nni_sock_add_dialer(nni_sock *s, nni_dialer *d)
 {
 	nni_sockopt *sopt;
-	int rv;
+	int          rv;
 
 	// grab a hold on the dialer for the socket
 	if ((rv = nni_dialer_hold(d)) != 0) {
@@ -1522,6 +1524,12 @@ nni_dialer_add_pipe(nni_dialer *d, void *tpipe)
 		nni_stat_inc(&d->st_reject, 1);
 		nni_stat_inc(&s->st_rejects, 1);
 #endif
+		if (nng_log_get_level() >= NNG_LOG_DEBUG) {
+			char addr[NNG_MAXADDRSTRLEN];
+			nng_log_debug("NNG-PIPEREJECT",
+			    "Pipe on socket<%u> from %s rejected by callback",
+			    nni_pipe_sock_id(p), nni_pipe_peer_addr(p, addr));
+		}
 		nni_pipe_rele(p);
 		return;
 	}
@@ -1541,6 +1549,12 @@ nni_dialer_add_pipe(nni_dialer *d, void *tpipe)
 	nni_stat_register(&p->st_root);
 #endif
 	nni_pipe_run_cb(p, NNG_PIPE_EV_ADD_POST);
+	if (nng_log_get_level() >= NNG_LOG_DEBUG) {
+		char addr[NNG_MAXADDRSTRLEN];
+		nng_log_debug("NNG-CONNECT",
+		    "Connected pipe<%u> on socket<%u> to %s", nni_pipe_id(p),
+		    nni_sock_id(s), nni_pipe_peer_addr(p, addr));
+	}
 	nni_pipe_rele(p);
 }
 
@@ -1651,6 +1665,12 @@ nni_listener_add_pipe(nni_listener *l, void *tpipe)
 	nni_stat_register(&p->st_root);
 #endif
 	nni_pipe_run_cb(p, NNG_PIPE_EV_ADD_POST);
+	if (nng_log_get_level() >= NNG_LOG_DEBUG) {
+		char addr[NNG_MAXADDRSTRLEN];
+		nng_log_debug("NNG-ACCEPT",
+		    "Accepted pipe<%u> on socket<%u> from %s", nni_pipe_id(p),
+		    nni_sock_id(s), nni_pipe_peer_addr(p, addr));
+	}
 	nni_pipe_rele(p);
 }
 
