@@ -1,5 +1,5 @@
 //
-// Copyright 2017 Garrett D'Amore <garrett@damore.org>
+// Copyright 2024 Garrett D'Amore <garrett@damore.org>
 // Copyright 2017 Capitar IT Group BV <info@capitar.com>
 //
 // This software is supplied under the terms of the MIT License, a
@@ -10,17 +10,32 @@
 
 #include "core/nng_impl.h"
 
+#include <time.h>
+
 #ifdef NNG_PLATFORM_WINDOWS
 
 nni_time
-nni_plat_clock(void)
+nni_clock(void)
 {
 	// We are limited by the system clock, but that is ok.
 	return (GetTickCount64());
 }
 
+int
+nni_time_get(uint64_t *seconds, uint32_t *nanoseconds)
+{
+	int             rv;
+	struct timespec ts;
+	if (timespec_get(&ts, TIME_UTC) == TIME_UTC) {
+		*seconds     = ts.tv_sec;
+		*nanoseconds = ts.tv_nsec;
+		return (0);
+	}
+	return (nni_win_error(GetLastError()));
+}
+
 void
-nni_plat_sleep(nni_duration dur)
+nni_msleep(nni_duration dur)
 {
 	uint64_t exp;
 
