@@ -1,5 +1,5 @@
 //
-// Copyright 2020 Staysail Systems, Inc. <info@staysail.tech>
+// Copyright 2021 Staysail Systems, Inc. <info@staysail.tech>
 // Copyright 2018 Capitar IT Group BV <info@capitar.com>
 // Copyright 2019 Devolutions <info@devolutions.net>
 //
@@ -39,12 +39,11 @@
 
 #ifdef NNG_SUPP_TLS
 
-static const nng_tls_engine *tls_engine;
-static nni_mtx               tls_engine_lock;
+static nni_atomic_ptr tls_engine;
 
 struct nng_tls_config {
 	nng_tls_engine_config_ops ops;
-	const nng_tls_engine *    engine; // store this so we can verify
+	const nng_tls_engine     *engine; // store this so we can verify
 	nni_mtx                   lock;
 	int                       ref;
 	int                       busy;
@@ -56,21 +55,21 @@ struct nng_tls_config {
 typedef struct {
 	nng_stream              stream;
 	nng_tls_engine_conn_ops ops;
-	nng_tls_config *        cfg;
-	const nng_tls_engine *  engine;
+	nng_tls_config         *cfg;
+	const nng_tls_engine   *engine;
 	size_t                  size;
-	nni_aio *               user_aio; // user's aio for connect/accept
+	nni_aio                *user_aio; // user's aio for connect/accept
 	nni_aio                 conn_aio; // system aio for connect/accept
 	nni_mtx                 lock;
 	bool                    closed;
 	bool                    hs_done;
 	nni_list                send_queue;
 	nni_list                recv_queue;
-	nng_stream *            tcp;      // lower level stream
+	nng_stream             *tcp;      // lower level stream
 	nni_aio                 tcp_send; // lower level send pending
 	nni_aio                 tcp_recv; // lower level recv pending
-	uint8_t *               tcp_send_buf;
-	uint8_t *               tcp_recv_buf;
+	uint8_t                *tcp_send_buf;
+	uint8_t                *tcp_recv_buf;
 	size_t                  tcp_recv_len;
 	size_t                  tcp_recv_off;
 	bool                    tcp_recv_pend;
@@ -102,7 +101,7 @@ static nni_reap_list tls_conn_reap_list = {
 typedef struct {
 	nng_stream_dialer  ops;
 	nng_stream_dialer *d; // underlying TCP dialer
-	nng_tls_config *   cfg;
+	nng_tls_config    *cfg;
 	nni_mtx            lk; // protects the config
 } tls_dialer;
 
@@ -131,7 +130,7 @@ tls_dialer_free(void *arg)
 static void
 tls_conn_cb(void *arg)
 {
-	tls_conn *  conn = arg;
+	tls_conn   *conn = arg;
 	nng_stream *tcp;
 	int         rv;
 
@@ -172,7 +171,7 @@ tls_dialer_dial(void *arg, nng_aio *aio)
 {
 	tls_dialer *d = arg;
 	int         rv;
-	tls_conn *  conn;
+	tls_conn   *conn;
 
 	if (nni_aio_begin(aio) != 0) {
 		return;
@@ -213,7 +212,7 @@ tls_dialer_set_config(void *arg, const void *buf, size_t sz, nni_type t)
 {
 	int             rv;
 	nng_tls_config *cfg;
-	tls_dialer *    d = arg;
+	tls_dialer     *d = arg;
 	nng_tls_config *old;
 
 	if ((rv = nni_copyin_ptr((void **) &cfg, buf, sz, t)) != 0) {
@@ -236,7 +235,7 @@ tls_dialer_set_config(void *arg, const void *buf, size_t sz, nni_type t)
 static int
 tls_dialer_get_config(void *arg, void *buf, size_t *szp, nni_type t)
 {
-	tls_dialer *    d = arg;
+	tls_dialer     *d = arg;
 	nng_tls_config *cfg;
 	int             rv;
 	nni_mtx_lock(&d->lk);
@@ -410,7 +409,7 @@ nni_tls_dialer_alloc(nng_stream_dialer **dp, const nng_url *url)
 typedef struct {
 	nng_stream_listener  ops;
 	nng_stream_listener *l;
-	nng_tls_config *     cfg;
+	nng_tls_config      *cfg;
 	nni_mtx              lk;
 } tls_listener;
 
@@ -446,7 +445,7 @@ tls_listener_accept(void *arg, nng_aio *aio)
 {
 	tls_listener *l = arg;
 	int           rv;
-	tls_conn *    conn;
+	tls_conn     *conn;
 
 	if (nni_aio_begin(aio) != 0) {
 		return;
@@ -470,7 +469,7 @@ tls_listener_set_config(void *arg, const void *buf, size_t sz, nni_type t)
 {
 	int             rv;
 	nng_tls_config *cfg;
-	tls_listener *  l = arg;
+	tls_listener   *l = arg;
 	nng_tls_config *old;
 
 	if ((rv = nni_copyin_ptr((void **) &cfg, buf, sz, t)) != 0) {
@@ -495,7 +494,7 @@ tls_listener_set_config(void *arg, const void *buf, size_t sz, nni_type t)
 static int
 tls_listener_get_config(void *arg, void *buf, size_t *szp, nni_type t)
 {
-	tls_listener *  l = arg;
+	tls_listener   *l = arg;
 	nng_tls_config *cfg;
 	int             rv;
 	nni_mtx_lock(&l->lk);
@@ -757,11 +756,51 @@ tls_get_verified(void *arg, void *buf, size_t *szp, nni_type t)
 	return (nni_copyout_bool(v, buf, szp, t));
 }
 
+static int
+tls_get_peer_cn(void *arg, void *buf, size_t *szp, nni_type t)
+{
+	NNI_ARG_UNUSED(szp);
+
+	if (t != NNI_TYPE_STRING) {
+		return (NNG_EBADTYPE);
+	}
+
+	tls_conn *conn = arg;
+	nni_mtx_lock(&conn->lock);
+	*(char **) buf = conn->ops.peer_cn((void *) (conn + 1));
+	nni_mtx_unlock(&conn->lock);
+	return (0);
+}
+
+static int
+tls_get_peer_alt_names(void *arg, void *buf, size_t *szp, nni_type t)
+{
+	NNI_ARG_UNUSED(szp);
+
+	if (t != NNI_TYPE_POINTER) {
+		return (NNG_EBADTYPE);
+	}
+
+	tls_conn *conn = arg;
+	nni_mtx_lock(&conn->lock);
+	*(char ***) buf = conn->ops.peer_alt_names((void *) (conn + 1));
+	nni_mtx_unlock(&conn->lock);
+	return (0);
+}
+
 static const nni_option tls_options[] = {
 	{
 	    .o_name = NNG_OPT_TLS_VERIFIED,
 	    .o_get  = tls_get_verified,
 	},
+	{
+	    .o_name = NNG_OPT_TLS_PEER_CN,
+	    .o_get  = tls_get_peer_cn,
+	},
+	{
+	    .o_name = NNG_OPT_TLS_PEER_ALT_NAMES,
+	    .o_get  = tls_get_peer_alt_names,
+	},
 	{
 	    .o_name = NULL,
 	},
@@ -770,7 +809,7 @@ static const nni_option tls_options[] = {
 static int
 tls_set(void *arg, const char *name, const void *buf, size_t sz, nni_type t)
 {
-	tls_conn *  conn = arg;
+	tls_conn   *conn = arg;
 	int         rv;
 	nng_stream *tcp;
 
@@ -798,7 +837,7 @@ tls_get(void *arg, const char *name, void *buf, size_t *szp, nni_type t)
 static int
 tls_alloc(tls_conn **conn_p, nng_tls_config *cfg, nng_aio *user_aio)
 {
-	tls_conn *            conn;
+	tls_conn             *conn;
 	const nng_tls_engine *eng;
 	size_t                size;
 
@@ -828,6 +867,7 @@ tls_alloc(tls_conn **conn_p, nng_tls_config *cfg, nng_aio *user_aio)
 	nni_aio_list_init(&conn->send_queue);
 	nni_aio_list_init(&conn->recv_queue);
 	nni_mtx_init(&conn->lock);
+	nni_aio_set_timeout(&conn->conn_aio, NNG_DURATION_INFINITE);
 	nni_aio_set_timeout(&conn->tcp_send, NNG_DURATION_INFINITE);
 	nni_aio_set_timeout(&conn->tcp_recv, NNG_DURATION_INFINITE);
 
@@ -870,6 +910,7 @@ tls_reap(void *arg)
 	if (conn->tcp_recv_buf != NULL) {
 		nni_free(conn->tcp_recv_buf, NNG_TLS_MAX_RECV_SIZE);
 	}
+	nni_mtx_fini(&conn->lock);
 	NNI_FREE_STRUCT(conn);
 }
 
@@ -1026,7 +1067,7 @@ static void
 tls_tcp_send_cb(void *arg)
 {
 	tls_conn *conn = arg;
-	nng_aio * aio  = &conn->tcp_send;
+	nng_aio  *aio  = &conn->tcp_send;
 	int       rv;
 	size_t    count;
 
@@ -1058,7 +1099,7 @@ static void
 tls_tcp_recv_cb(void *arg)
 {
 	tls_conn *conn = arg;
-	nni_aio * aio  = &conn->tcp_recv;
+	nni_aio  *aio  = &conn->tcp_recv;
 	int       rv;
 
 	nni_mtx_lock(&conn->lock);
@@ -1233,9 +1274,9 @@ nng_tls_config_cert_key_file(
     nng_tls_config *cfg, const char *path, const char *pass)
 {
 	int    rv;
-	void * data;
+	void  *data;
 	size_t size;
-	char * pem;
+	char  *pem;
 
 	if ((rv = nni_file_get(path, &data, &size)) != 0) {
 		return (rv);
@@ -1255,9 +1296,9 @@ int
 nng_tls_config_ca_file(nng_tls_config *cfg, const char *path)
 {
 	int    rv;
-	void * data;
+	void  *data;
 	size_t size;
-	char * pem;
+	char  *pem;
 
 	if ((rv = nni_file_get(path, &data, &size)) != 0) {
 		return (rv);
@@ -1357,7 +1398,7 @@ nng_tls_config_auth_mode(nng_tls_config *cfg, nng_tls_auth_mode mode)
 int
 nng_tls_config_alloc(nng_tls_config **cfg_p, nng_tls_mode mode)
 {
-	nng_tls_config *      cfg;
+	nng_tls_config       *cfg;
 	const nng_tls_engine *eng;
 	size_t                size;
 	int                   rv;
@@ -1366,9 +1407,7 @@ nng_tls_config_alloc(nng_tls_config **cfg_p, nng_tls_mode mode)
 		return (rv);
 	}
 
-	nni_mtx_lock(&tls_engine_lock);
-	eng = tls_engine;
-	nni_mtx_unlock(&tls_engine_lock);
+	eng = nni_atomic_get_ptr(&tls_engine);
 
 	if (eng == NULL) {
 		return (NNG_ENOTSUP);
@@ -1424,9 +1463,8 @@ nng_tls_engine_name(void)
 	const nng_tls_engine *eng;
 
 	nni_init();
-	nni_mtx_lock(&tls_engine_lock);
-	eng = tls_engine;
-	nni_mtx_unlock(&tls_engine_lock);
+
+	eng = nni_atomic_get_ptr(&tls_engine);
 
 	return (eng == NULL ? "none" : eng->name);
 }
@@ -1437,9 +1475,8 @@ nng_tls_engine_description(void)
 	const nng_tls_engine *eng;
 
 	nni_init();
-	nni_mtx_lock(&tls_engine_lock);
-	eng = tls_engine;
-	nni_mtx_unlock(&tls_engine_lock);
+
+	eng = nni_atomic_get_ptr(&tls_engine);
 
 	return (eng == NULL ? "" : eng->description);
 }
@@ -1450,9 +1487,8 @@ nng_tls_engine_fips_mode(void)
 	const nng_tls_engine *eng;
 
 	nni_init();
-	nni_mtx_lock(&tls_engine_lock);
-	eng = tls_engine;
-	nni_mtx_unlock(&tls_engine_lock);
+
+	eng = nni_atomic_get_ptr(&tls_engine);
 
 	return (eng == NULL ? false : eng->fips_mode);
 }
@@ -1461,11 +1497,12 @@ int
 nng_tls_engine_register(const nng_tls_engine *engine)
 {
 	if (engine->version != NNG_TLS_ENGINE_VERSION) {
+		nng_log_err("NNG-TLS-ENGINE-VER",
+		    "TLS Engine version mismatch: %d != %d", engine->version,
+		    NNG_TLS_ENGINE_VERSION);
 		return (NNG_ENOTSUP);
 	}
-	nni_mtx_lock(&tls_engine_lock);
-	tls_engine = engine;
-	nni_mtx_unlock(&tls_engine_lock);
+	nni_atomic_set_ptr(&tls_engine, (void *) engine);
 	return (0);
 }
 
@@ -1492,12 +1529,9 @@ int
 nni_tls_sys_init(void)
 {
 	int rv;
-	nni_mtx_init(&tls_engine_lock);
-	tls_engine = NULL;
 
 	rv = NNG_TLS_ENGINE_INIT();
 	if (rv != 0) {
-		nni_mtx_fini(&tls_engine_lock);
 		return (rv);
 	}
 	return (0);
@@ -1506,7 +1540,6 @@ nni_tls_sys_init(void)
 void
 nni_tls_sys_fini(void)
 {
-	nni_reap_drain();
 	NNG_TLS_ENGINE_FINI();
 }
 
@@ -1614,6 +1647,12 @@ nng_tls_config_alloc(nng_tls_config **cfgp, nng_tls_mode mode)
 	return (NNG_ENOTSUP);
 }
 
+void
+nng_tls_config_hold(nng_tls_config *cfg)
+{
+	NNI_ARG_UNUSED(cfg);
+}
+
 void
 nng_tls_config_free(nng_tls_config *cfg)
 {
@@ -1692,4 +1731,4 @@ nni_tls_sys_fini(void)
 {
 }
 
-#endif // !NNG_SUPP_TLS
\ No newline at end of file
+#endif // !NNG_SUPP_TLS
