@@ -1,5 +1,5 @@
 //
-// Copyright 2021 Staysail Systems, Inc. <info@staysail.tech>
+// Copyright 2024 Staysail Systems, Inc. <info@staysail.tech>
 //
 // This software is supplied under the terms of the MIT License, a
 // copy of which should be located in the distribution where this
@@ -17,9 +17,18 @@
 #ifndef NNG_TESTING_NUTS_H
 #define NNG_TESTING_NUTS_H
 
+#include <nng/nng.h>
+extern void nuts_logger(
+    nng_log_level, nng_log_facility, const char *, const char *);
+
 // Call nng_fini during test finalization -- this avoids leak warnings.
 extern void nng_fini(void);
 #define TEST_FINI nng_fini()
+#define TEST_INIT                                 \
+	do {                                      \
+		nng_log_set_logger(nuts_logger);  \
+		nng_log_set_level(NNG_LOG_DEBUG); \
+	} while (0)
 #include "acutest.h"
 
 #include <stdbool.h>
@@ -27,7 +36,6 @@ extern void nng_fini(void);
 #include <string.h>
 
 // The following headers are provided for test code convenience.
-#include <nng/nng.h>
 #include <nng/protocol/bus0/bus.h>
 #include <nng/protocol/pair0/pair.h>
 #include <nng/protocol/pair1/pair.h>
@@ -101,6 +109,7 @@ extern const char *nuts_server_key;
 extern const char *nuts_server_crt;
 extern const char *nuts_client_key;
 extern const char *nuts_client_crt;
+extern const char *nuts_garbled_crt;
 
 // NUTS_SUCCESS tests for NNG success.  It reports the failure if it
 // did not.
@@ -203,6 +212,17 @@ extern const char *nuts_client_crt;
 
 #define NUTS_PROTO(x, y) (((x) << 4u) | (y))
 
+#define NUTS_ENABLE_LOG(level)                         \
+	do {                                           \
+		nng_log_set_logger(nng_stderr_logger); \
+		nng_log_set_level(level);              \
+	} while (0)
+
+#define NUTS_LOGGING()                            \
+	do {                                      \
+		nng_log_set_logger(nuts_logger);  \
+		nng_log_set_level(NNG_LOG_DEBUG); \
+	} while (0)
 #ifdef __cplusplus
 };
 #endif
